@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001 Ananova Ltd
- * Copyright 2002,2006,2007,2008,2009,2010,2011,2012,2015,2016 Olly Betts
+ * Copyright 2002,2006,2007,2008,2009,2010,2011,2012,2015,2016,2018 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -40,6 +40,10 @@
 
 using namespace std;
 
+// HTML5 legacy compatibility doctype.
+#define HTML5_LEGACY_COMPAT "about:legacy-compat"
+#define HTML5_LEGACY_COMPAT_LEN CONST_STRLEN(HTML5_LEGACY_COMPAT)
+
 inline void
 lowercase_string(string &str)
 {
@@ -224,9 +228,11 @@ HtmlParser::parse(const string &body)
 
 	if (*start == '!') {
 	    if (++start == body.end()) break;
+
+	    // Comment, SGML declaration, or HTML5 DTD.
+	    char first_ch = *start;
 	    if (++start == body.end()) break;
-	    // comment or SGML declaration
-	    if (*(start - 1) == '-' && *start == '-') {
+	    if (first_ch == '-' && *start == '-') {
 		++start;
 		string::const_iterator close = find(start, body.end(), '>');
 		// An unterminated comment swallows rest of document
@@ -261,8 +267,9 @@ HtmlParser::parse(const string &body)
 		    // Otherwise skip to the first > we found (as Netscape does).
 		    start = close;
 		}
-	    } else if (body.size() - (start - body.begin()) > 6 &&
-		       body.compare(start - body.begin() - 1, 7, "[CDATA[", 7) == 0) {
+	    } else if (first_ch == '[' &&
+		       body.size() - (start - body.begin()) > 6 &&
+		       body.compare(start - body.begin(), 6, "CDATA[", 6) == 0) {
 		start += 6;
 		string::size_type b = start - body.begin();
 		string::size_type i;
@@ -272,8 +279,71 @@ HtmlParser::parse(const string &body)
 		process_text(text);
 		if (i == string::npos) break;
 		start = body.begin() + i + 2;
+	    } else if (C_tolower(first_ch) == 'd' &&
+		       body.end() - start > 6 &&
+		       C_tolower(start[0]) == 'o' &&
+		       C_tolower(start[1]) == 'c' &&
+		       C_tolower(start[2]) == 't' &&
+		       C_tolower(start[3]) == 'y' &&
+		       C_tolower(start[4]) == 'p' &&
+		       C_tolower(start[5]) == 'e' &&
+		       C_isspace(start[6])) {
+		// DOCTYPE declaration.
+		start += 7;
+		while (start != body.end() && C_isspace(*start)) {
+		    ++start;
+		}
+		if (start == body.end()) break;
+		if (body.end() - start >= 5 &&
+		    C_tolower(start[0]) == 'h' &&
+		    C_tolower(start[1]) == 't' &&
+		    C_tolower(start[2]) == 'm' &&
+		    C_tolower(start[3]) == 'l' &&
+		    (start[4] == '>' || C_isspace(start[4]))) {
+		    start += 4;
+
+		    // HTML doctype.
+		    while (start != body.end() && C_isspace(*start)) {
+			++start;
+		    }
+		    if (start == body.end()) break;
+
+		    if (*start == '>') {
+			// <!DOCTYPE html>
+			// Default charset for HTML5 is UTF-8.
+			charset = "utf-8";
+		    }
+		} else if (body.end() - start >= 29 &&
+			   C_tolower(start[0]) == 's' &&
+			   C_tolower(start[1]) == 'y' &&
+			   C_tolower(start[2]) == 's' &&
+			   C_tolower(start[3]) == 't' &&
+			   C_tolower(start[4]) == 'e' &&
+			   C_tolower(start[5]) == 'm' &&
+			   C_isspace(start[6])) {
+		    start += 7;
+		    while (start != body.end() && C_isspace(*start)) {
+			++start;
+		    }
+		    size_t left = body.end() - start;
+		    if (left >= HTML5_LEGACY_COMPAT_LEN + 3 &&
+			(*start == '\'' || *start == '"') &&
+			start[HTML5_LEGACY_COMPAT_LEN + 1] == *start &&
+			body.compare(start - body.begin() + 1,
+				     HTML5_LEGACY_COMPAT_LEN,
+				     HTML5_LEGACY_COMPAT,
+				     HTML5_LEGACY_COMPAT_LEN) == 0) {
+			// HTML5 legacy compatibility doctype:
+			// <!DOCTYPE html SYSTEM "about:legacy-compat">
+			start += HTML5_LEGACY_COMPAT_LEN + 2;
+			// Default charset for HTML5 is UTF-8.
+			charset = "utf-8";
+		    }
+		}
+		start = find(start - 1, body.end(), '>');
+		if (start == body.end()) break;
 	    } else {
-		// just an SGML declaration, perhaps giving the DTD - ignore it
+		// Some other SGML declaration - ignore it.
 		start = find(start - 1, body.end(), '>');
 		if (start == body.end()) break;
 	    }
