@@ -1,7 +1,7 @@
 /** @file runfilter.cc
  * @brief Run an external filter and capture its output in a std::string.
  */
-/* Copyright (C) 2003,2006,2007,2009,2010,2011,2013,2015 Olly Betts
+/* Copyright (C) 2003,2006,2007,2009,2010,2011,2013,2015,2017 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -359,7 +359,10 @@ use_shell_after_all:
 	argv.push_back(NULL);
 
 	execvp(argv[0], const_cast<char **>(&argv[0]));
-	_exit(-1);
+	// Emulate shell behaviour and exit with status 127 if the command
+	// isn't found, and status 126 for other problems.  In particular, we
+	// rely on 127 below to throw NoSuchFilter.
+	_exit(errno == ENOENT ? 127 : 126);
     }
 
     // We're the parent process.
@@ -387,7 +390,7 @@ use_shell_after_all:
 	int r = select(fd + 1, &readfds, NULL, NULL, &tv);
 	if (r <= 0) {
 	    if (r < 0) {
-		if (errno == EINTR) {
+		if (errno == EINTR || errno == EAGAIN) {
 		    // select() interrupted by a signal, so retry.
 		    continue;
 		}
